#!/usr/bin/env python3

import os
import nlopt
import scipy.optimize as optimize

# data from: http://hea.iki.rssi.ru/AZT22/RUS/eaa_bessel.pdf (with a correction of K-band)
# Johnson R and I passbands from Sterken & Manfroid (1992), p. 234
# OAO-2 filters from Kondo et al. (1994)

# photometric filters
filters = [
    'johnson_u',
    'johnson_b',
    'johnson_v', 
    'johnson_r',
    'johnson_i',
    'johnson_j',
    'johnson_h',
    'johnson_k',
    'johnson_l',
    'johnson_m',
    'johnson_n',
    'oao2_3320',
    'oao2_2980',
    'oao2_2460',
    'oao2_1910',
    'oao2_1550',
    'oao2_1430',
    'iue_1365',
    'iue_1250',
    ]

# effective wavelengths [m]
eff_wave = [
    367.e-9,
    436.e-9,
    545.e-9,
    720.e-9,
    900.e-9,
    1220.e-9,
    1630.e-9,
    2190.e-9,
    3450.e-9,
    4750.e-9,
    10500.e-9,
    332.e-9,
    298.e-9,
    246.e-9,
    191.e-9,
    155.e-9,
    143.e-9,
    136.5e-9,
    125.0e-9,
    ]

# effective bandpasses [m]
# The last one (IUE) is uncertain! In Kondo et al. (1994), there is either 50, or 100 Ang.
eff_band = [
    66.e-9,
    94.e-9,
    85.e-9,
    220.e-9,
    240.e-9,
    213.e-9,
    307.e-9,
    390.e-9,
    472.e-9,
    460.e-9,
    2500.e-9,
    52.e-9,
    41.e-9,
    36.e-9,
    26.e-9,
    27.e-9,
    24.e-9,
    1.e-9,
    5.e-9,
    ]

# calibration fluxes F_lambda for 0.0 mag [ J s^-1 m^-2 m^-1 ]
# UBVRIJKLMN from Johnson (1965)
# H from ftp://ftp.astro.ufl.edu/pub/wilson/lcdc2015/ebdoc.13july2015.pdf
# OAO-2 from Kondo et al. (1994)
# IUE's calibration fluxes were NOT given in Kondo et al. (1994), but I computed them from Flux & Mag in Tables 3A, 3B
calibration_flux = [
    0.0435,
    0.0688,
    0.0378,
    0.0185,
    0.00899,
    0.00340,
    0.00039,
    0.001133,
    0.000080,
    0.000022,
    0.00000124,
    3.17e-3,
    3.22e-3,
    3.28e-3,
    6.47e-3,
    5.88e-3,
    6.88e-3,
    6.39931211478635e-3,
    5.09529872628506e-3,
    ]

# Note: erg s^-1 cm^-2 Ang^-1 = 10^-7 J s^-1 10^4 m^-2 10^10 m^-1 = 10^7 J s^1 m^-2 m^-1

# The same, just two records are given for each
# lattitude and longitude in degrees.
location_definitions = [
        ['chara', 'npoi'],
        [34.223759, 35.0959],
        [-118.061645, -111.5339]
    ]

# definition of fitting environmernts
fitter_definitions = dict(
    nlopt_esch=dict(par0type='value',
                    optional_kwarsg=['xtol', 'ftol', 'maxfun'],
                    object=None,
                    environment=nlopt.GN_ESCH,
                    uses_bounds=True,
                    info='This is a modified Evolutionary Algorithm for global optimization, '
                    'developed by Carlos Henrique da Silva Santos.'),
    nlopt_nelder_mead=dict(par0type='value',
                           optional_kwargs=['xtol', 'ftol', 'maxfun'],
                           object=None,
                           environment=nlopt.LN_NELDERMEAD,
                           uses_bounds=True,
                           info='Nelder-Mead Simplex. Implementation NLOPT: Steven G. Johnson, '
                                'The NLopt nonlinear-optimization package, http://ab-initio.mit.edu/nlopt.'),
    nlopt_sbplx=dict(par0type='value',
                     optional_kwargs=['xtol', 'ftol', 'maxfun'],
                     object=None,
                     environment=nlopt.LN_SBPLX,
                     uses_bounds=True,
                     info='Sbplx - a variation of the Tom Rowans Subplex. '
                          'Implementation NLOPT: Steven G. Johnson, The NLopt '
                          'nonlinear-optimization package, http://ab-initio.mit.edu/nlopt.'),
    sp_diff_evol=dict(par0type='bounds',
                      optional_kwargs=['tol'],
                      object=None,
                      environment=optimize.differential_evolution,
                      uses_bounds=True,
                      info='The algorithm is due to Storn & Price (1997).',
                    ),
)


# Here parameters of shellspec objects are defined.
object_definitions = {
    'environmental': [
        dict(name='imodel', value=1, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='irotat', value=1, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='ipart', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='ichemc', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='ielnd', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='ithom', value=1, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='irayl', value=1, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='imie', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='imiepf', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='ihyd', value=1, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='iopac', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='iline', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='eps', value=0, unit=None, vmin=0, vmax=1000, fitted=False, dtype=float, docstring=''),
        dict(name='ionu', value=1, unit=None, vmin=1, vmax=1000, fitted=False, dtype=int, docstring=''),
        dict(name='ior', value=1, unit=None, vmin=1, vmax=1000, fitted=False, dtype=int, docstring=''),
        dict(name='iot', value=1, unit=None, vmin=1, vmax=1000, fitted=False, dtype=int, docstring=''),
        dict(name='offset', value=1.0, unit=None, vmin=-1000, vmax=1000, fitted=False, dtype=float, docstring=''),
        dict(name='lunt1', value=1, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='lunt2', value=1, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='lunt3', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
    ],
    'grid': [
        dict(name='rmdfx1', value=-80, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfx2', value=80, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfy1', value=-80, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfy2', value=80, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfz1', value=-30, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfz2', value=30, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepfx', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepfy', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepfz', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainfx', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainfy', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainfz', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfx3', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfx4', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfy3', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfy4', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfz3', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdfz4', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepfxb', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepfyb', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepfzb', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainfxb', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainfyb', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainfzb', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdx1', value=-80, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdx2', value=80, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdy1', value=-30, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdy2', value=30, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdz1', value=-80, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdz2', value=80, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepx', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepy', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepz', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainx', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainy', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainz', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdx3', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdx4', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdy3', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdy4', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdz3', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rmdz4', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepxb', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepyb', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='stepzb', value=1.00, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainxb', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainyb', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='gainzb', value=1., unit=None, vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
    ],
    'distance' : [
        dict(name='dd', value=294., unit='parsec', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vgamma', value=0.0, unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='iext', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='rv', value=3.1, unit=None, vmin=0., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='ebv', value=0.3226, unit='mag', vmin=0., vmax=10., fitted=False, dtype=float, docstring=''),
    ],
    'orbit': [
        dict(name='period', value=12.913779, unit='day', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''), 
        dict(name='t0', value=2408254.4248895, unit='day', vmin=0., vmax=1000000000., fitted=False, dtype=float, docstring=''),  # i.e. half a period LATER than Ak etal.
        dict(name='qeph', value=3.87265e-6, unit=None, vmin=0., vmax=10., fitted=False, dtype=float, docstring=''), 
        dict(name='sma', value=58.349, unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='q', value=0.2230, unit=None, vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='ecc', value=0.0, unit=None, vmin=0., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='dinc', value=88.0, unit='deg', vmin=0., vmax=180., fitted=False, dtype=float, docstring=''),
        dict(name='omega', value=90., unit='deg', vmin=-360., vmax=360., fitted=False, dtype=float, docstring=''),
        dict(name='omega_an', value=254., unit='deg', vmin=-360., vmax=360., fitted=False, dtype=float, docstring=''),
        dict(name='asini', value=58.19, unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        ],
    'central_object': [
        dict(name='istar', value=1, unit=None, vmin=0, vmax=2, fitted=False, dtype=int, docstring=''),
        dict(name='rstar', value=5.987, unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='tstar', value=30000., unit='Kelvin', vmin=1000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='emstar', value=13.048, unit='solMass', vmin=0.01, vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='xstar', value=0., unit=None, vmin=-1.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='ystar', value=0., unit=None, vmin=-1.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='zstar', value=1., unit=None, vmin=-1.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='vrotst', value=200., unit='km / s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='idifst', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='drotst', value=0.0, unit=None, vmin=0., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='hst', value=0.0, unit=None, vmin=0., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='vxst', value=0.0, unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vyst', value=0.0, unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vzst', value=0.0, unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='dlst', value=0.26, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='dlst2', value=0.0, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='dgst', value=0.25, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='ffst', value=0.00, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='irrst', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='ialbst', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='albst', value=0.3, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='htst', value=0.6, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='htsta', value=1.0, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='ispst', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='xspst', value=-1.0, unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='yspst', value=0.0, unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='zspst', value=1.0, unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='aspst', value=1.0, unit='deg', vmin=0., vmax=360., fitted=False, dtype=float, docstring=''),
        dict(name='tspst', value=0.9, unit=None, vmin=0., vmax=1., fitted=False, dtype=float, docstring=''),
        ],
    'companion': [
        dict(name='icomp', value=2, unit=None, vmin=0, vmax=2, fitted=False, dtype=int, docstring=''),
        dict(name='rcp', value=19.84, unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='tempcp', value=13300., unit='Kelvin', vmin=10., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='qq', value=0.223, unit=None, vmin=0.00, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='vrxcp', value=0., unit=None, vmin=-1.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='vrycp', value=0., unit=None, vmin=-1.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='vrzcp', value=1., unit=None, vmin=-1.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='vrotcp', value=0., unit='km / s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='xcp', value=58.349, unit='solRad', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='ycp', value=0.0, unit='solRad', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='zcp', value=0.0, unit='solRad', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vxcp', value=0.0, unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vycp', value=0.0, unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vzcp', value=0.0, unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='dlcp', value=0.36, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='dlcp2', value=0.0, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='dgcp', value=0.25, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='ffcp', value=1.00, unit=None, vmin=0.0, vmax=10.0, fitted=False, dtype=float, docstring=''),
        dict(name='irrcp', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='ialbcp', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='albcp', value=0.3, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='htcp', value=0.6, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='htcpa', value=1.0, unit=None, vmin=0.0, vmax=1.0, fitted=False, dtype=float, docstring=''),
        dict(name='ispst', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring='')
        ],
    'envelope': [
        dict(name='ienv', value=2, unit=None, vmin=0, vmax=3, fitted=False, dtype=int, docstring=''),
        dict(name='emen', value=13.048, unit='solMass', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='qqen', value=0.2230, unit=None, vmin=0., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='aen', value=58.349, unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='ffen', value=1.0, unit=None, vmin=0., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='hen', value=8.0, unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='tempen', value=9000., unit='Kelvin', vmin=0., vmax=1000000., fitted=False, dtype=float, docstring=''),
        dict(name='densen', value=1.0e-9, unit=None, vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='aneen', value=21e9, unit=None, vmin=0., vmax=1.0e20, fitted=False, dtype=float, docstring=''),
        dict(name='vtrben', value=0., unit='km /s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='dstden', value=0., unit=None, vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='dstten', value=0., unit='Kelvin', vmin=0., vmax=1000000., fitted=False, dtype=float, docstring=''),
        ],
    'disk': [
        dict(name='idisc', value=1, unit=None, vmin=0, vmax=3, fitted=False, dtype=int, docstring=''),
        dict(name='adisc', value=12., unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='rindc', value=6., unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='routdc', value=30., unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='emdc', value=13.049, unit='solMass', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='rdc', value=5.987, unit='solMass', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='xdc', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='ydc', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='zdc', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='xdisc', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='ydisc', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='zdisc', value=1., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='vxdc', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vydc', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vzdc', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='tempdc', value=20000., unit='Kelvin', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='densdc', value=1.0e-7, unit=None, vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='anedc', value=12.e9, unit=None, vmin=0., vmax=1.e20, fitted=False, dtype=float, docstring=''),
        dict(name='vtrbdc', value=10., unit='km / s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='edendc', value=-1., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='itdc', value=1, unit=None, vmin=0, vmax=3, fitted=False, dtype=int, docstring=''),
        dict(name='etmpdc', value=-0.85, unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dstddc', value=0., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dsttdc', value=1000., unit='Kelvin', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        ],
    'ufo': [
        dict(name='iufo', value=1, unit=None, vmin=0, vmax=3, fitted=False, dtype=int, docstring=''),
        dict(name='aufo', value=12., unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='rinuf', value=6., unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='routuf', value=30., unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='emuf', value=13.049, unit='solMass', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='ruf', value=5.987, unit='solMass', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='xuf', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='yuf', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='zuf', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='xdisc', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='ydisc', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='zdisc', value=1., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='vxuf', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vyuf', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vzuf', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='tempuf', value=20000., unit='Kelvin', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='densuf', value=1.0e-7, unit=None, vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='aneuf', value=12.e9, unit=None, vmin=0., vmax=1.e20, fitted=False, dtype=float, docstring=''),
        dict(name='vtrbuf', value=10., unit='km / s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='edenuf', value=-1., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='ituf', value=1, unit=None, vmin=0, vmax=3, fitted=False, dtype=int, docstring=''),
        dict(name='etmpuf', value=-0.85, unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dstduf', value=0., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dsttuf', value=1000., unit='Kelvin', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        ],
    'spot': [
        dict(name='ispot', value=1, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='vrxsp', value=0., unit=None, vmin=0., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='vrysp', value=0., unit=None, vmin=0., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='vrzsp', value=1., unit=None, vmin=0., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='vrotsp', value=0., unit='km / s', vmin=0., vmax=1000000., fitted=False, dtype=float, docstring=''),
        dict(name='rsp', value=1., unit='solRad', vmin=0., vmax=1000000., fitted=False, dtype=float, docstring=''),
        dict(name='xsp', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='ysp', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='zsp', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vxsp', value=0., unit='km / s', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vysp', value=0., unit='km / s', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vzsp', value=0., unit='km / s', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='tempsp', value=30000., unit='Kelvin', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='denssp', value=1.e-8, unit=None, vmin=0., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='anesp', value=21.e9, unit=None, vmin=0., vmax=1.e12, fitted=False, dtype=float, docstring=''),
        dict(name='vtrbsp', value=0., unit='km / s', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='dstdsp', value=0., unit=None, vmin=0., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dsttsp', value=3000., unit='Kelvin', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rpolsp', value=0., unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vpolsp', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='pangsp', value=0., unit='deg', vmin=0., vmax=720., fitted=False, dtype=float, docstring=''),
        ],
    'nebula': [
        dict(name='inebl', value=1, unit=None, vmin=0, vmax=4, fitted=False, dtype=int, docstring=''),
        dict(name='aneb', value=7., unit=None, vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='rinnb', value=6., unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='routnb', value=30., unit='solRad', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='emnb', value=13.049, unit='solMass', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='rnb', value=5.987, unit='solMass', vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='iinvnb', value=1, unit=None, vmin=0, vmax=2, fitted=False, dtype=int, docstring=''),
        dict(name='hinvnb', value=6.0, unit=None, vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='tinvnb', value=6.0, unit=None, vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='hwindnb', value=8.8, unit=None, vmin=0., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='idennb', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='hcnb', value=1.0, unit=None, vmin=0., vmax=100., fitted=False, dtype=float, docstring=''),
        dict(name='ivelnb', value=0, unit=None, vmin=0, vmax=1, fitted=False, dtype=int, docstring=''),
        dict(name='hvelnb', value=1.0, unit=None, vmin=0., vmax=100., fitted=False, dtype=float, docstring=''),
        dict(name='vnb', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='evelnb', value=1.0, unit=None, vmin=-2.0, vmax=2.0, fitted=False, dtype=float, docstring=''),
        dict(name='ishdnb', value=1, unit=None, vmin=0, vmax=3, fitted=False, dtype=int, docstring=''),
        dict(name='hshdnb', value=1.0, unit=None, vmin=0., vmax=100., fitted=False, dtype=float, docstring=''),
        dict(name='xneb', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='yneb', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='zneb', value=1., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='vxnb', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vynb', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vznb', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='tempnb', value=20000., unit='Kelvin', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='densnb', value=1.0e-7, unit=None, vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='anenb', value=12.e9, unit=None, vmin=0., vmax=1.e20, fitted=False, dtype=float, docstring=''),
        dict(name='vtrbnb', value=10., unit='km / s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='edennb', value=-1., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='itnb', value=1, unit=None, vmin=0, vmax=3, fitted=False, dtype=int, docstring=''),
        dict(name='etmpnb', value=-0.85, unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dstdnb', value=0., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dsttnb', value=1000., unit='Kelvin', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        ],
    'jet': [
        dict(name='ijet', value=2, unit=None, vmin=0, vmax=2, fitted=False, dtype=int, docstring=''),
        dict(name='ajet', value=10.0, unit='deg', vmin=0, vmax=90.0, fitted=False, dtype=float, docstring=''),
        dict(name='rinjt', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='routjt', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='ivjt', value=2, unit=None, vmin=0, vmax=2, fitted=False, dtype=int, docstring=''),
        dict(name='vjt', value=500., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='eveljt', value=1., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='rcjt', value=1.9, unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='xjt', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='yjt', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='zjt', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vxjt', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vyjt', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vzjt', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='xjet', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='yjet', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='zjet', value=1., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='tempjt', value=20000., unit='Kelvin', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='densjt', value=1.0e-7, unit=None, vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='anejt', value=12.e9, unit=None, vmin=0., vmax=1.e20, fitted=False, dtype=float, docstring=''),
        dict(name='vtrbjt', value=10., unit='km / s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='dstdjt', value=0., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dsttjt', value=1000., unit='Kelvin', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='etmpjt', value=0., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='asymjt', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='rpoljt', value=0., unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vpoljt', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='pangjt', value=0., unit='deg', vmin=-360., vmax=360., fitted=False, dtype=float, docstring=''),
        ],
    'flow': [
        dict(name='iflow', value=1, unit=None, vmin=0, vmax=4, fitted=False, dtype=int, docstring=''),
        dict(name='v1fw', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='v2fw', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='r1fw', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='r2fw', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='x1fw', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='y1fw', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='z1fw', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='x2fw', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='y2fw', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='z2fw', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vxfw', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vyfw', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vzfw', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='xfw', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='yfw', value=0., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='zfw', value=1., unit=None, vmin=-1., vmax=1., fitted=False, dtype=float, docstring=''),
        dict(name='pfw', value=1.e30, unit='day', vmin=0., vmax=1.e30, fitted=False, dtype=float, docstring=''),
        dict(name='tempfw', value=20000., unit='Kelvin', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='densfw', value=1.0e-7, unit=None, vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='anefw', value=12.e9, unit=None, vmin=0., vmax=1.e20, fitted=False, dtype=float, docstring=''),
        dict(name='vtrbfw', value=10., unit='km / s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='edenfw', value=0., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dstdfw', value=0., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dsttfw', value=1000., unit='Kelvin', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='rpolfw', value=0., unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='pangfw', value=0., unit='deg', vmin=0., vmax=720., fitted=False, dtype=float, docstring=''),
        dict(name='r12fw', value=0., unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='z12fw', value=0., unit='solRad', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='v12fw', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        ],
    'shell': [
        dict(name='ishell', value=3, unit=None, vmin=0, vmax=3, fitted=False, dtype=int, docstring=''),
        dict(name='rinsh', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='routsh', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vsh', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='evelsh', value=1.0, unit=None, vmin=-2.0, vmax=2.0, fitted=False, dtype=float, docstring=''),
        dict(name='rcsh', value=0., unit='solRad', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='vxsh', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vysh', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='vzsh', value=0., unit='km / s', vmin=-10000., vmax=10000., fitted=False, dtype=float, docstring=''),
        dict(name='tempsh', value=20000., unit='Kelvin', vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='denssh', value=1.0e-7, unit=None, vmin=0., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='anesh', value=12.e9, unit=None, vmin=0., vmax=1.e20, fitted=False, dtype=float, docstring=''),
        dict(name='vtrbsh', value=10., unit='km / s', vmin=0., vmax=1000., fitted=False, dtype=float, docstring=''),
        dict(name='dstdsh', value=0., unit=None, vmin=-10., vmax=10., fitted=False, dtype=float, docstring=''),
        dict(name='dsttsh', value=1000., unit='Kelvin', vmin=-100000., vmax=100000., fitted=False, dtype=float, docstring=''),
        dict(name='etmpsh', value=0.0, unit=None, vmin=-2., vmax=2., fitted=False, dtype=float, docstring=''),
        ],
    }

# definition of addresses of shellspec files are stored
path = os.path.realpath(__file__)
shsp_template = os.path.join('/'.join(path.split('/')[:-1]), 'include/shellspec/template.in') 
shsp_executable = os.path.join('/'.join(path.split('/')[:-1]), 'include/shellspec/shellspec')
shsp_params = os.path.join('/'.join(path.split('/')[:-1]), 'include/shellspec/params.inc')
shsp_abundance = os.path.join('/'.join(path.split('/')[:-1]), 'include/shellspec/abundances')

